import { globalLookup } from "@/hook/global";
import { reactive } from "vue";
import { contextmenu } from "./handle-contextmenu";

import i18n from '@/i18n/index';
import { ElMessage } from "element-plus";
import { findViewIndexByLink } from "@/hook/wave-container-view";

const { t } = i18n.global;

// 从 globalLookup.currentWiresRenderView 中可以获取当前的分组情况
export function getCurrentGroups() {
    const groups = globalLookup.currentWiresRenderView.filter(view => view.renderType === 1);
    return groups;
}

export const existGroup = reactive({
    display: false,
    show() {
        this.display = true;
    },
    hide() {
        this.display = false;
    }
});


/**
 * 
 * @param {WaveRenderSidebarItem} group 
 * @param {WireItem} signal 
 */
function inGroup(group, signal) {
    for (const child of group.children) {
        if (child.signalInfo.link === signal.link) {
            return true;
        }
    }
    return false;
}

/**
 * 
 * @param {WaveRenderSidebarItem} group 
 */
export function joinGroup(group) {
    const signal = contextmenu.currentWire;
    if (signal === undefined) {
        return;
    }

    // 检查是否已经在里面了
    if (inGroup(group, signal)) {
        ElMessage({
            message: t('context-menu.cannot-join-repeat-group'),
            type: 'warning',
            plain: true,
            showClose: true,
            duration: 0
        });
        return;
    }

    // 加入 group 中
    group.children.push({
        signalInfo: {
            name: signal.name,
            link: signal.link,
            size: signal.size,
        },
        groupInfo: {
            name: '',
            color: '',
            collapse: false
        },
        renderType: 0,
        children: []
    });

    // 删除的逻辑参照 wave-container-view.js delete 函数
    const renderView = globalLookup.currentWiresRenderView;
    const i = findViewIndexByLink(signal.link);
    const tailElements = renderView.slice(i + 1);
    renderView.length = i;
    tailElements.forEach(view => renderView.push(view));
}